package com.tistory.antop.proftpd.common;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import com.tistory.antop.proftpd.common.util.Utils;
import com.tistory.antop.proftpd.common.vo.FilterVo;
import com.tistory.antop.proftpd.common.vo.InquireInVo;
import com.tistory.antop.proftpd.common.vo.MapVo;
import com.tistory.antop.proftpd.common.vo.SortVo;

public class BaseSo {
	// log4j
	protected Logger logger = Logger.getLogger(getClass());

	protected void mapping(InquireInVo inVo, MapVo mapVo) {
		mapping(inVo, mapVo, null);
	}

	protected void mapping(InquireInVo inVo, MapVo mapVo, MapVo convertProp) {
		renameProperty(inVo, convertProp);
		mappingSort(inVo.getSort(), mapVo);
		mappingFilter(inVo.getFilter(), mapVo);

		mapVo.put("start", inVo.getStart());
		mapVo.put("limit", inVo.getLimit());
	}

	/**
	 * 프로퍼티명을 설정한 것으로 바꿔준다
	 * 프로퍼티와 바뀌어질 칼럼명이 매칭되지 않을 경우 이 메소드에서 설정 된다.
	 * createDt 프로퍼티는 create_dt 로 바뀌어진다.
	 * 
	 * 바뀌기 바로 직전에!
	 * 
	 * 자바의 name 프로퍼티가 db 에서는 g_name 이랑 매핑 되는 경우.
	 * renameProperty 에 name = gName 으로 등록하면 된다.
	 * 그러면 프로퍼티명이 바뀌어서 등록 되고 gName 은 g_name 으로 다시 변환된다. 
	 * 
	 * Mybatis resultMap 에서 매핑 되는것과는 별개로
	 * Extjs 의 필터와 소팅으로 넘어오는 프로퍼티를 매핑하기 위함이다.
	 * 
	 * @param inVo
	 * @param renameProperty
	 */
	private void renameProperty(InquireInVo inVo, MapVo renameProperty) {
		if (renameProperty == null || renameProperty.size() == 0) {
			return;
		}

		for (SortVo sortVo : inVo.getSort()) {
			String property = sortVo.getProperty();

			if (renameProperty.containsKey(property)) {
				sortVo.setProperty(renameProperty.getString(property));
			}
		}

		for (FilterVo filterVo : inVo.getFilter()) {
			String property = filterVo.getProperty();

			if (renameProperty.containsKey(property)) {
				filterVo.setProperty(renameProperty.getString(property));
			}
		}

	}

	/**
	 * SortVo
	 * 
	 * @param sortVos
	 * @param mapVo
	 */
	private void mappingSort(List<SortVo> sortVos, MapVo mapVo) {
		if (sortVos == null || sortVos.size() == 0) {
			return;
		}

		List<MapVo> sorts = new ArrayList<MapVo>();

		for (SortVo sortVo : sortVos) {
			String property = sortVo.getProperty();
			String direction = sortVo.getDirection();

			if (Utils.isEmpty(property)) {
				continue;
			}

			if (Utils.isEmpty(direction)) {
				direction = "asc";
			}

			MapVo sort = new MapVo();
			
			// 여기서 프로퍼티의 낙타 표기법이 언더바 표기법으로 바뀐다.
			sort.put("property", Utils.convertCamelToUnder(property));
			sort.put("direction", direction);

			sorts.add(sort);
		}

		mapVo.put("sorts", sorts);
	}

	private void mappingFilter(List<FilterVo> filterVos, MapVo mapVo) {
		List<MapVo> filters = new ArrayList<MapVo>();

		for (FilterVo vo : filterVos) {
			String operator = vo.getOperator();
			String type = vo.getType();
			Object value = vo.getValue();

			MapVo filter = new MapVo();

			if ("int".equalsIgnoreCase(type)) {
				filter.put("value", Integer.parseInt(value.toString()));
			} else if ("bool".equalsIgnoreCase(type)) {
				filter.put("value", parseBoolean(value.toString()));
			} else if ("list".equals(type)) {
				filter.put("value", value);
			} else {
				filter.put("value", value);
			}
			// determine the operation
			if ("lt".equalsIgnoreCase(operator)) {
				filter.put("op", "<");
			} else if ("gt".equalsIgnoreCase(operator)) {
				filter.put("op", ">");
			} else if ("eq".equalsIgnoreCase(operator)) {
				filter.put("op", "=");
			} else {
				filter.put("op", operator);
			}

			filter.put("type", vo.getType());
			// 낙타표기법 -> 언더바 포기법
			filter.put("prop", Utils.convertCamelToUnder(vo.getProperty()));

			filters.add(filter);
		}

		mapVo.put("filters", filters);

	}

	private boolean parseBoolean(String s) {
		if (s == null) {
			return false;
		}

		if (s.equalsIgnoreCase("true")) {
			return true;
		}

		if (s.equalsIgnoreCase("false")) {
			return false;
		}

		try {
			if (Integer.parseInt(s) > 1) {
				return true;
			} else {
				return false;
			}
		} catch (NumberFormatException e) {
			return false;
		}

	}

}
