package com.tistory.antop.proftpd.common.password;

import java.security.GeneralSecurityException;
import java.security.MessageDigest;

import org.springframework.security.authentication.encoding.PasswordEncoder;

/**
 * MYSQL password() 알고리즘을 이용한 비밀번호 암호화
 * 
 * @author Antop
 *
 */
public class MySqlPasswordEncoder implements PasswordEncoder {

	@Override
	public String encodePassword(String rawPass, Object salt) {
		if (rawPass == null) {
			throw new NullPointerException();
		}

		byte[] bpara = new byte[rawPass.length()];

		byte[] rethash;
		int i;
		for (i = 0; i < rawPass.length(); i++)
			bpara[i] = (byte) (rawPass.charAt(i) & 0xff);
		try {
			MessageDigest sha1er = MessageDigest.getInstance("SHA1");
			rethash = sha1er.digest(bpara); // stage1
			rethash = sha1er.digest(rethash); // stage2
		} catch (GeneralSecurityException e) {
			throw new RuntimeException(e);
		}
		StringBuffer r = new StringBuffer(41);
		r.append("*");
		for (i = 0; i < rethash.length; i++) {
			String x = Integer.toHexString(rethash[i] & 0xff).toUpperCase();
			if (x.length() < 2)
				r.append("0");
			r.append(x);
		}

		return r.toString();
	}

	@Override
	public boolean isPasswordValid(String encPass, String rawPass, Object salt) {

		if (encPass == null || rawPass == null) {
			return false;
		}

		if (!encPass.equals(encodePassword(rawPass, salt))) {
			return false;
		}

		return true;
	}

}
