package com.tistory.antop.proftpd.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import com.tistory.antop.proftpd.common.BaseController;
import com.tistory.antop.proftpd.common.command.ListCommand;
import com.tistory.antop.proftpd.common.vo.InquireOutVo;
import com.tistory.antop.proftpd.controller.command.GroupJoinCommand;
import com.tistory.antop.proftpd.controller.command.GroupSaveCommand;
import com.tistory.antop.proftpd.mybatis.model.Group;
import com.tistory.antop.proftpd.service.GroupSo;
import com.tistory.antop.proftpd.service.vo.GroupInquireVo;
import com.tistory.antop.proftpd.service.vo.GroupSaveVo;

@Controller
public class GroupController extends BaseController {

	@Autowired
	private GroupSo groupSo;

	/**
	 * 그룹 목록 조회
	 * @param modelMap
	 * @param command
	 * @return
	 */
	
	@RequestMapping(value = "/group/list", method = RequestMethod.POST)
	@ResponseBody
	public ModelMap list(ModelMap modelMap, @RequestBody ListCommand command) {

		GroupInquireVo inVo = new GroupInquireVo();

		mapping(command, inVo);

		InquireOutVo<Group> outVo = groupSo.inquire(inVo);

		modelMap.put("results", outVo.getList());
		modelMap.put("total", outVo.getTotal());

		return modelMap;
	}

	/**
	 * 그룹 정보 저장
	 * @param modelMap
	 * @param command
	 * @return
	 */
	@RequestMapping(value = "/group/save", method = RequestMethod.POST)
	@ResponseBody
	public ModelMap save(ModelMap modelMap,
			@RequestBody GroupSaveCommand command) {

		GroupSaveVo saveVo = new GroupSaveVo();
		saveVo.setOriginalName(command.getOriginalName());
		saveVo.setName(command.getName());
		saveVo.setGid(command.getGid());

		Group group = groupSo.save(saveVo);

		modelMap.put("group", group);

		return modelMap;
	}

	/**
	 * 그룹 삭제
	 * @param modelMap
	 * @param names 그룹[]
	 * @return
	 */
	@RequestMapping(value = "/group/remove", method = RequestMethod.POST)
	@ResponseBody
	public ModelMap remove(ModelMap modelMap, @RequestBody List<String> names) {
		groupSo.remove(names);

		return modelMap;
	}

	/**
	 * 사용자들을 그룹에 포함 시킨다.
	 * @param modelMap
	 * @param command 사용자[] / 그룹명
	 * @return
	 */
	@RequestMapping(value = "/group/join", method = RequestMethod.POST)
	@ResponseBody
	public ModelMap join(ModelMap modelMap,
			@RequestBody GroupJoinCommand command) {

		groupSo.join(command.getGroupName(), command.getUsers());

		return modelMap;
	}

	/**
	 * 사용자들을 그룹에 제외 시킨다.
	 * @param modelMap
	 * @param users 사용자[]
	 * @return
	 */
	@RequestMapping(value = "/group/leave", method = RequestMethod.POST)
	@ResponseBody
	public ModelMap leave(ModelMap modelMap, @RequestBody List<String> users) {

		groupSo.leave(users);

		return modelMap;
	}

}
