package com.tistory.antop.proftpd.mybatis.handler;

import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.ibatis.type.JdbcType;
import org.apache.ibatis.type.TypeHandler;

import com.tistory.antop.proftpd.common.util.Utils;

/**
 * 자바 클래스의 List<String>와 데이터베이스의 콤마 구분자를 매핑한다.
 * 
 * @author Antop
 * 
 */
public class ListCommaStringHandler implements TypeHandler<List<String>> {

	@Override
	public List<String> getResult(ResultSet rs, String columnName)
			throws SQLException {
		return convertCommaStringToList(rs.getString(columnName));
	}

	@Override
	public List<String> getResult(ResultSet rs, int columnIndex)
			throws SQLException {
		return convertCommaStringToList(rs.getString(columnIndex));
	}

	@Override
	public List<String> getResult(CallableStatement cs, int columnIndex)
			throws SQLException {
		return convertCommaStringToList(cs.getString(columnIndex));
	}

	@Override
	public void setParameter(PreparedStatement pstmt, int columnIndex,
			List<String> list, JdbcType jdbcType) throws SQLException {
		pstmt.setString(columnIndex, convertListToCommaString(list));
	}

	private List<String> convertCommaStringToList(String commaString) {
		List<String> list = new ArrayList<String>();

		if (commaString == null) {
			return list;
		}

		StringTokenizer st = new StringTokenizer(commaString, ",");
		while (st.hasMoreTokens()) {
			String s = st.nextToken();
			if (Utils.isEmpty(s)) {
				continue;
			}

			list.add(s.trim());
		}

		return list;
	}

	private String convertListToCommaString(List<String> list) {
		String commaString = "";

		if (list == null || list.size() == 0) {
			return null;
		}

		int size = list.size();
		for (int i = 0; i < size; i++) {
			String s = list.get(i);

			if (Utils.isNotEmpty(s)) {
				if (i > 0) {
					commaString += ",";
				}
				commaString += s.trim();
			}

		}

		return commaString;
	}
}
