package com.tistory.antop.proftpd.mybatis.model;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import com.tistory.antop.proftpd.common.util.Utils;

public class User implements UserDetails {

	private static final long serialVersionUID = -6788461192266338742L;

	// 고유 아이디
	private Long id;
	// 유저 아이디
	private String userid;
	// 비밀번호
	private String passwd;
	// 이메일
	private String email;
	// UID
	private Integer uid;
	// GID
	private Integer gid;
	// HOME DIR
	private String homedir;
	// SHELL
	private String shell;
	// 접속 수
	private int count = 0;
	// 마지막 접속일시
	private Date accessed;
	// 계정 유효 여부
	private boolean valid = true;
	// 스프링 시큐리티에서 사용하는 역할
	private String authority;

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public Integer getUid() {
		return uid;
	}

	public Integer getGid() {
		return gid;
	}

	public String getPasswd() {
		return passwd;
	}

	public void setPasswd(String passwd) {
		this.passwd = passwd;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public String getShell() {
		return shell;
	}

	public void setShell(String shell) {
		this.shell = shell;
	}

	public String getHomedir() {
		return homedir;
	}

	public void setHomedir(String homedir) {
		this.homedir = homedir;
	}

	public int getCount() {
		return count;
	}

	public void setCount(int count) {
		this.count = count;
	}

	public String getUserid() {
		return userid;
	}

	public void setUserid(String userid) {
		this.userid = userid;
	}

	public Date getAccessed() {
		return accessed;
	}

	public void setAccessed(Date accessed) {
		this.accessed = accessed;
	}

	public void setUid(Integer uid) {
		this.uid = uid;
	}

	public void setGid(Integer gid) {
		this.gid = gid;
	}

	public boolean isValid() {
		return valid;
	}

	public void setValid(boolean valid) {
		this.valid = valid;
	}

	public String getAuthority() {
		return Utils.isNotEmpty(authority) ? authority : null;
	}

	public void setAuthority(String authority) {
		this.authority = authority;
	}

	@Override
	public String toString() {
		return "User [id=" + id + ", userid=" + userid + ", passwd=" + passwd
				+ ", email=" + email + ", uid=" + uid + ", gid=" + gid
				+ ", homedir=" + homedir + ", shell=" + shell + ", count="
				+ count + ", accessed=" + accessed + ", valid=" + valid
				+ ", authority=" + authority + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((userid == null) ? 0 : userid.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		User other = (User) obj;
		if (userid == null) {
			if (other.userid != null)
				return false;
		} else if (!userid.equals(other.userid))
			return false;
		return true;
	}

	@Override
	public Collection<? extends GrantedAuthority> getAuthorities() {
		List<Authority> authorities = new ArrayList<User.Authority>();
		authorities.add(new Authority(this.authority));
		return authorities;
	}

	@Override
	public String getPassword() {
		return this.passwd;
	}

	@Override
	public String getUsername() {
		return this.userid;
	}

	@Override
	public boolean isAccountNonExpired() {
		return this.valid;
	}

	@Override
	public boolean isAccountNonLocked() {
		return this.valid;
	}

	@Override
	public boolean isCredentialsNonExpired() {
		return this.valid;
	}

	@Override
	public boolean isEnabled() {
		return this.valid;
	}

	class Authority implements GrantedAuthority {

		private static final long serialVersionUID = -7117380774042216122L;

		private String authority;

		public Authority(String authority) {
			this.authority = authority;
		}

		@Override
		public String getAuthority() {
			return this.authority;
		}

	}
}
