package com.tistory.antop.proftpd.service.impl;

import java.util.List;

import org.springframework.transaction.annotation.Transactional;

import com.tistory.antop.proftpd.common.BaseSo;
import com.tistory.antop.proftpd.common.util.Utils;
import com.tistory.antop.proftpd.common.vo.InquireOutVo;
import com.tistory.antop.proftpd.common.vo.MapVo;
import com.tistory.antop.proftpd.mybatis.mapper.GroupMapper;
import com.tistory.antop.proftpd.mybatis.model.Group;
import com.tistory.antop.proftpd.service.GroupSo;
import com.tistory.antop.proftpd.service.vo.GroupInquireVo;
import com.tistory.antop.proftpd.service.vo.GroupSaveVo;

public class GroupSoImpl extends BaseSo implements GroupSo {

	private GroupMapper groupMapper;

	public void setGroupMapper(GroupMapper groupMapper) {
		this.groupMapper = groupMapper;
	}

	@Override
	public InquireOutVo<Group> inquire(GroupInquireVo inVo) {

		InquireOutVo<Group> outVo = new InquireOutVo<Group>();

		MapVo mapVo = new MapVo();

		MapVo renameProperty = new MapVo();
		renameProperty.put("name", "groupname");

		mapping(inVo, mapVo, renameProperty);

		List<Group> list = groupMapper.selectByMap(mapVo);
		int total = groupMapper.countByMap(mapVo);

		return outVo.set(list, total);
	}

	@Override
	@Transactional
	public Group save(GroupSaveVo saveVo) {
		boolean update = true;

		// 입력된 그룹명으로 조회
		Group group = groupMapper.selectByPrimaryKey(saveVo.getName());
		// 일단 검색이 안되면 등록
		if (group == null) {
			group = new Group();
			group.setName(saveVo.getName());
			group.setOriginalName(saveVo.getOriginalName());

			if (Utils.isEmpty(group.getOriginalName())) { // insert 경우
				update = false;
			}
		} else {
			// 그룹명 변경/그룹명 추가 때에 그룹명 중복 확인
			if (saveVo.getOriginalName() != null
					&& saveVo.getName().equals(saveVo.getOriginalName()) == false) {
				throw new RuntimeException("등록 되어있는 그룹명입니다.");
			}

			group.setOriginalName(group.getName());
		}

		group.setGid(saveVo.getGid());

		if (update) {
			groupMapper.update(group);
		} else {
			groupMapper.insert(group);
		}

		return group;
	}

	@Override
	@Transactional
	public void remove(List<String> names) {
		for (String name : names) {
			groupMapper.delete(name);
		}
	}

	@Override
	@Transactional
	public void join(String groupName, List<String> users) {
		// 그룹에 유저를 포함
		Group group = groupMapper.selectByPrimaryKey(groupName);

		for (String userId : users) {
			if (group.getMembers().contains(userId) == false) {
				group.getMembers().add(userId);
			}

			leave(userId);
		}

		// update
		groupMapper.update(group);
	}

	@Override
	public void leave(List<String> users) {
		for (String userId : users) {
			leave(userId);
		}
	}

	// 다른 그룹에 포함되어 있는 유저 제거
	private void leave(String userId) {
		List<Group> groups = groupMapper.selectInUser(userId);
		for (Group g : groups) {
			g.getMembers().remove(userId);
			// update
			groupMapper.update(g);
		}
	}
}