package com.tistory.antop.proftpd.service.impl;

import java.util.List;

import org.springframework.security.authentication.encoding.PasswordEncoder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.transaction.annotation.Transactional;

import com.tistory.antop.proftpd.common.BaseSo;
import com.tistory.antop.proftpd.common.util.Utils;
import com.tistory.antop.proftpd.common.vo.InquireOutVo;
import com.tistory.antop.proftpd.common.vo.MapVo;
import com.tistory.antop.proftpd.mybatis.mapper.UserMapper;
import com.tistory.antop.proftpd.mybatis.model.User;
import com.tistory.antop.proftpd.mybatis.model.UserInGroup;
import com.tistory.antop.proftpd.service.UserSo;
import com.tistory.antop.proftpd.service.vo.UserInGroupInquireVo;
import com.tistory.antop.proftpd.service.vo.UserInquireVo;
import com.tistory.antop.proftpd.service.vo.UserSaveVo;

public class UserSoImpl extends BaseSo implements UserSo, UserDetailsService {

	private UserMapper userMapper;
	private PasswordEncoder passwordEncoder;

	public void setUserMapper(UserMapper userMapper) {
		this.userMapper = userMapper;
	}

	public void setPasswordEncoder(PasswordEncoder passwordEncoder) {
		this.passwordEncoder = passwordEncoder;
	}

	@Override
	public User inquire(String userId) {
		return userMapper.selectByPrimaryKey(userId);
	}

	@Override
	public InquireOutVo<User> inquire(UserInquireVo inVo) {

		InquireOutVo<User> outVo = new InquireOutVo<User>();

		MapVo mapVo = new MapVo();

		mapping(inVo, mapVo, null);

		List<User> list = userMapper.selectByMap(mapVo);
		int total = userMapper.countByMap(mapVo);

		return outVo.set(list, total);
	}

	@Override
	@Transactional
	public User save(UserSaveVo saveVo) {

		User user = new User();

		mapping(saveVo, user);

		if (Utils.isNotEmpty(user.getPasswd())) {
			user.setPasswd(passwordEncoder.encodePassword(user.getPasswd(),
					null));
		}

		if (user.getId() == null) {
			userMapper.insert(user);
		} else {
			userMapper.update(user);
		}

		user.setPasswd(null);

		return user;
	}

	@Override
	@Transactional
	public void remove(List<Long> ids) {
		if (ids == null || ids.size() == 0) {
			throw new RuntimeException("삭제할 유저를 선택하세요");
		}

		userMapper.delete(ids);
	}

	private void mapping(UserSaveVo saveVo, User user) {
		user.setId(saveVo.getId());
		user.setUserid(saveVo.getUserid());
		if (Utils.isNotEmpty(saveVo.getPasswd())) {
			user.setPasswd(saveVo.getPasswd());
		}
		user.setEmail(saveVo.getEmail());
		user.setValid(saveVo.getValid());
		user.setAuthority(saveVo.getAuthority());
	}

	public InquireOutVo<UserInGroup> inquireByGroup(UserInGroupInquireVo inVo) {

		InquireOutVo<UserInGroup> outVo = new InquireOutVo<UserInGroup>();

		MapVo mapVo = new MapVo();

		mapping(inVo, mapVo, null);

		List<UserInGroup> list = userMapper.selectInGroup(mapVo);
		int total = userMapper.countByMap(mapVo);

		return outVo.set(list, total);
	}

	@Override
	public UserDetails loadUserByUsername(String username)
			throws UsernameNotFoundException {

		User user = userMapper.selectByPrimaryKey(username);

		if (user == null) {
			throw new UsernameNotFoundException(username);
		}

		return user;
	}
}
