// loader
Ext.Loader.setConfig({
	enabled : true,
	disableCaching: true,
	paths : {
		'Ext.ux' : CONTEXT_PATH + '/js/extjs-4.1.0/ux',
		'Proftpd' : CONTEXT_PATH + '/js/proftpd'
	}
});

/**
 * 모든 AJAX 공통
 */
Ext.require('Ext.Ajax', function() {
	
	Ext.apply(Ext.Ajax, {
		// 메소드는 POST
		method : 'post',
		// json 으로 데이터를 보낸다.
		defaultHeaders : {
			'Content-Type' : 'application/json; charset=UTF-8'
		}
	});
	
	Ext.Ajax.on({
		// 요청 하기 전 특정 파라미터 json 인코딩
		beforerequest : function(conn, o) {
			var params = o.params;
			
			// 넘어오는 데이터중에 sort 와 filter 부분은 문자열로 넘어오게 되는데 json object 로 변환한다
			if(Ext.isObject(params)) {
				// grid sort
				if(params.sort && Ext.isObject(params.sort) == false) {
					params.sort = Ext.decode(params.sort);
				}
				// grid filter
				if(params.filter && Ext.isObject(params.filter) == false) {
					params.filter = Ext.decode(params.filter);
				}
			}
			
			// 전체를 json 문자열로 변환
			o.params = Ext.encode(params);
			
//			console.log(o.params);
		},
		// 서버 에러
		requestexception : function(conn, response, option) {
			Ext.MessageBox.show({
				title : response.status + ' ' + response.statusText,
				buttons : Ext.Msg.OK,
				width : 500,
				msg : response.responseText
			});
		},
		requestcomplete : function(conn, resp, options) {
			var json = Ext.decode(resp.responseText);
			
			if (json.success == false) {
				switch (json.error.code) {
				case 'L':
					// 로그인되지 않음

					if(Ext.getCmp('Proftpd.LoginWindow')) {
						return;
					}
					
					var win = Ext.create('Proftpd.LoginWindow', {
						req : {
							conn : conn,
							options : options
						}
					});
					
					win.show();
					
					break;
				default:
					Ext.Msg.alert('Server Error', json.error.message);

					break;
				}
				
			}
			
		}

	});
	
});

Ext.require('Ext.data.AbstractStore', function() {
	Ext.override(Ext.data.AbstractStore, {

	    //method can by overwritten to allow some specific behaviour during load state change
	    setLoadingState:function(loading, operation){
	        this.loading = loading;
	    },

	    /**
	     * Loads the Store using its configured {@link #proxy}.
	     * @param {Object} options (optional) config object. This is passed into the {@link Ext.data.Operation Operation}
	     * object that is created and then sent to the proxy's {@link Ext.data.proxy.Proxy#read} function
	     */
	    //FIX 1: adding method setLoadingState() to manage store load states
	    load: function(options) {
	        var me = this,
	            operation;

	        options = options || {};

	        Ext.applyIf(options, {
	            action : 'read',
	            filters: me.filters.items,
	            sorters: me.getSorters()
	        });

	        operation = Ext.create('Ext.data.Operation', options);

	        if (me.fireEvent('beforeload', me, operation) !== false) {
	            //FIX 1
	            me.setLoadingState(true, operation);
	            me.proxy.read(operation, me.onProxyLoad, me);
	        }

	        return me;
	    }
	});
});