/*
*  Paging support added
*
*  FIXME 1: tree nodes are deleted BEFORE operation is confirmed, see FIXME 2
*  FIXME 2: synchronize this fix with FIXME 2, nodes should be removed from store here
*  FIXME 3: strange bug, workaround to show nodes properly after sync() when some nodes were moved (added) to saving node with children
*           - just collapse node (hide childred) and allow users to expand it manually after operation is done
* */
Ext.define('Ext.ux.data.TreePagingStore', {
	extend : 'Ext.data.TreeStore',
	
    /**
     * @cfg {Number} pageSize
     * The number of records considered to form a 'page'. This is used to power the built-in
     * paging using the nextPage and previousPage functions. Defaults to 25.
     */
    pageSize: 25,

    //overwrite AbstractStore.sync()
    sync: function() {
        var me        = this,
            options   = {},
            toCreate  = me.getNewRecords(),
            toUpdate  = me.getUpdatedRecords(),
            toDestroy = me.getRemovedRecords(),
            needsSync = false;

        if (toCreate.length > 0) {
            options.create = toCreate;
            needsSync = true;
        }

        if (toUpdate.length > 0) {
            options.update = toUpdate;
            needsSync = true;
        }

        if (toDestroy.length > 0) {
            options.destroy = toDestroy;
            needsSync = true;
        }

        if (needsSync && me.fireEvent('beforesync', options) !== false) {
            //FIXME 3 -- begin
            if (options.create) {
                Ext.Array.forEach(options.create,function(i,idx,a){
                    var n;
                    if (i.parentNode) {
                        n = i.parentNode;
                    } else {
                        n = i;
                    }
                    if (n.isRoot()) {
                        n.collapseChildren(true);
                    } else {
                        n.collapse(true);
                    }
                });
            }
            if (options.update) {
                Ext.Array.forEach(options.update,function(i,idx,a){
                    var n;
                    if (i.parentNode) {
                        n = i.parentNode;
                    } else {
                        n = i;
                    }
                    if (n.isRoot()) {
                        n.collapseChildren(true);
                    } else {
                        n.collapse(true);
                    }
                });
            }
            // -- end of FIX 3
            me.proxy.batch(options, me.getBatchListeners());
        }
    },

    //method added in AbstractStore
    setLoadingState:function(loading, operation){
        var me = this,
            node = operation ? operation.node : null;
        if (loading == true) {
            if (node) {
                node.set('loading', true);
            }
        } else {
            if (node) {
                node.set('loading', false);
            }
        }
        me.callParent(arguments);
    },

    //Cleanup of new, modified and deleted marks when node is refreshed
    //private
    nodeRefreshCleanup:function(node){
        var me = this;
        if (me.clearOnLoad) {
            me.removeChildNodes(node, false);
        }
    },
    //removes all childs recurrently
    //private
    removeChildNodes:function(node, destroy, suppressEvents) {
        var cn = node.childNodes,
            n;

        while ((n = cn[0])) {
            this.removeChildNodes(n, destroy, suppressEvents);
            // prevent being added to the removed cache
            n.isReplace = true;
            node.removeChild(n, destroy, suppressEvents);
            delete n.isReplace;
        }
        return node;
    },
    /**
     * Loads the Store using its configured {@link #proxy}.
     * @param {Object} options (Optional) config object. This is passed into the {@link Ext.data.Operation Operation}
     * object that is created and then sent to the proxy's {@link Ext.data.proxy.Proxy#read} function.
     * The options can also contain a node, which indicates which node is to be loaded. If not specified, it will
     * default to the root node.
     */
    //overwrite
    load: function(options) {
        options = options || {};
        options.params = options.params || {};

        var me = this,
            node = options.node || me.tree.getRootNode(), currentPage;

        // If there is not a node it means the user hasnt defined a rootnode yet. In this case lets just
        // create one for them.
        if (!node) {
            node = me.setRootNode({
                expanded: true
            });
        }

//------
        //FIXME 1
//        if (me.clearOnLoad) {
//            node.removeAll(true);
//        }
        currentPage = node.currentPage || 1;
        Ext.applyIf(options, {
            node: node,
            page: currentPage,
            start: (currentPage - 1) * me.pageSize,
            limit: me.pageSize,
            addRecords: false
        });
//------

        options.params[me.nodeParam] = node ? node.getId() : 'root';

        return me.callParent([options]);
    },
    // inherit docs
    //overwrite
    onProxyLoad: function(operation) {
        var me = this,
            resultSet = operation.getResultSet(),
            successful = operation.wasSuccessful(),
            records = operation.getRecords(),
            node = operation.node,
            currentPage = operation.page || 1;

        me.setLoadingState(false, operation);

        if (successful) {
//------
            //FIXME 2
            me.nodeRefreshCleanup(node);
            if (resultSet) {
                node.currentPage = currentPage;
                node.totalCount = resultSet.total;
            }
            if(node.isRoot() && !node.isExpanded()) {
                node.data.loaded = true;
                node.expand(false);
            }
//------
            records = me.fillNode(node, records);

        }
        // The load event has an extra node parameter
        // (differing from the load event described in AbstractStore)
        /**
         * @event load
         * Fires whenever the store reads data from a remote data source.
         * @param {Ext.data.TreeStore} this
         * @param {Ext.data.NodeInterface} node The node that was loaded.
         * @param {Ext.data.Model[]} records An array of records.
         * @param {Boolean} successful True if the operation was successful.
         */
        // deprecate read?
        me.fireEvent('read', me, operation.node, records, successful);
        me.fireEvent('load', me, operation.node, records, successful);
        //this is a callback that would have been passed to the 'read' function and is optional
        Ext.callback(operation.callback, operation.scope || me, [records, operation, successful]);
    },


//********************************************************
//               Methods added for paging
//********************************************************

    /**
     * Returns the total number of {@link Ext.data.Model Model} instances that the {@link Ext.data.proxy.Proxy Proxy}
     * indicates exist. This will usually differ from {@link #getCount} when using paging - getCount returns the
     * number of records loaded into the Store at the moment, getTotalCount returns the number of records that
     * could be loaded into the Store if the Store contained all data
     * @return {Number} The total number of Model instances available via the Proxy
     */
    getTotalCount: function(node) {
        return node ? node.totalCount : (this.getRootNode().totalCount || 0);
    },
    // PAGING METHODS
    /**
     * Loads a given 'page' of data by setting the start and limit values appropriately. Internally this just causes a normal
     * load operation, passing in calculated 'start' and 'limit' params
     * @param {Number} page The number of the page to load
     * @param {Object} options See options for {@link #load}
     */
    loadPage: function(page, options) {
        var me = this;
        options = Ext.apply({}, options),
        node = options.node || me.tree.getRootNode();

        page = page || 1;
        me.read(Ext.applyIf(options, {
            node: node,
            page: page,
            start: (page - 1) * me.pageSize,
            limit: me.pageSize,
            addRecords: !me.clearOnLoad
        }));
    },
    /**
     * Loads the next 'page' in the current data set
     * @param {Object} options See options for {@link #load}
     */
    nextPage: function(options) {
        this.loadPage((options.node&&options.node.currentPage?options.node.currentPage:1) + 1, options);
    },
    /**
     * Loads the previous 'page' in the current data set
     * @param {Object} options See options for {@link #load}
     */
    previousPage: function(options) {
        this.loadPage((options.node&&options.node.currentPage?options.node.currentPage:1) - 1, options);
    },
    clearFilter: function(suppressEvent) {
        var me = this;

        me.filters.clear();

        if (me.remoteFilter) {

            // In a buffered Store, the meaing of suppressEvent is to simply clear the filters collection
            if (suppressEvent) {
                return;
            }

            // For a buffered Store, we have to clear the prefetch cache because the dataset will change upon filtering.
            // Then we must prefetch the new page 1, and when that arrives, reload the visible part of the Store
            // via the guaranteedrange event
            if (me.buffered) {
                me.pageMap.clear();
                me.loadPage(1);
            } else {
                // Reset to the first page, clearing a filter will destroy the context of the current dataset
                me.currentPage = 1;
                me.load();
            }
        } else if (me.isFiltered()) {
            me.data = me.snapshot.clone();
            delete me.snapshot;

            if (suppressEvent !== true) {
                me.fireEvent('datachanged', me);
                me.fireEvent('refresh', me);
            }
        }
    },
    filter : function (filters, value) {
        if (Ext.isString(filters)) {
            filters = {
                property: filters,
                value: value
            };
        }

        var me = this,
            decoded = me.decodeFilters(filters),
            i = 0,
            doLocalSort = me.sorters.length && me.sortOnFilter && !me.remoteSort,
            length = decoded.length;

        for (; i < length; i++) {
            me.filters.replace(decoded[i]);
        }

        if (me.remoteFilter) {
            // For a buffered Store, we have to clear the prefetch cache because the dataset will change upon filtering.
            // Then we must prefetch the new page 1, and when that arrives, reload the visible part of the Store
            // via the guaranteedrange event
            if (me.buffered) {
                me.pageMap.clear();
                me.loadPage(1);
            } else {
                // Reset to the first page, the filter is likely to produce a smaller data set
                me.currentPage = 1;
                //the load function will pick up the new filters and request the filtered data from the proxy
                me.load();
            }
        } else {
            /**
             * @property {Ext.util.MixedCollection} snapshot
             * A pristine (unfiltered) collection of the records in this store. This is used to reinstate
             * records when a filter is removed or changed
             */
            if (me.filters.getCount()) {
                me.snapshot = me.snapshot || me.data.clone();
                me.data = me.data.filter(me.filters.items);

                if (doLocalSort) {
                    me.sort();
                } else {
                    // fire datachanged event if it hasn't already been fired by doSort
                    me.fireEvent('datachanged', me);
                    me.fireEvent('refresh', me);
                }
            }
        }
    }
}, function() {
    Ext.regStore('ext-empty-treestore', {
    	proxy: 'proxy'
    });
});