Ext.define('Proftpd.GroupGrid', {
	id : 'Proftpd.GroupGrid',
	extend : 'Ext.grid.Panel',
	uses : [
		'Ext.ux.PagingToolbarResizer',
		'Ext.ux.CheckColumn',
		'Ext.ux.form.field.ClearButton',
		'Ext.ux.grid.feature.MultiSorting'
	],
	constructor : function() {
		var me = this;

		// model
		me.createModel();
		// store
		me.createStore();
		// selection model
		me.createSelectionModel();
		// top toolbar
		me.createTopToolbar();
		// bottom bar
		me.createBottomToolbar();
		// filter bar plugin
		me.createFilterBar();
		// cell editing
		me.createCellEditor();
		
		Ext.apply(me, {
			border : false,
			store : me.store,
			viewConfig: {
	            stripeRows: true
	        },
	        selModel: me.sm,
			columns : [ {
				flex : 1,
				text : '그룹명',
				dataIndex : 'name',
				width : 100,
				filter : {
					type : 'string'
				},
				field: {
	                allowBlank: false
	            }
			}, {
				text : '그룹 식별자',
				dataIndex : 'gid',
				width : 100,
				filter : {
					type : 'int'
				},
				field: {
					xtype : 'numberfield',
					minValue : 0,
	                allowBlank: false
	            }
			}],
			// 순서 주의!
			plugins : [ this.filterBar, this.cellEditor ],
			features : [ Ext.create('Ext.ux.grid.feature.MultiSorting') ]
		});

		// create
		me.callParent(arguments);
		
		me.on({
			scope : this,
			render : function() {
				me.store.load();
			}
		});

	},
	createModel : function() {
		// model
		Ext.define('Group', {
			extend : 'Ext.data.Model',
			idProperty : 'name',
			fields : [ 'name', 'gid', 'members' ]
		});
	},
	createStore : function() {
		var me = this;
		
		me.store = Ext.create('Ext.data.Store', {
			model : 'Group',
			pageSize : 50,
			proxy : {
				type : 'ajax',
				url : CONTEXT_PATH + '/group/list',
				reader : {
					type : 'json',
					root : 'results',
					totalProperty : 'total'
				},
				actionMethods : {
					read : 'post'
				}
			},
			remoteSort : true,
			remoteFilter : true
		});
		
		me.store.on({
			beforeload : function() {
				if(!me.body.isMasked()) {
					me.body.mask('로딩중...');
				}
			},
			load : function() {
				if(me.body.isMasked()) {
					me.body.unmask();
				}
			}
		});
		
		return me.store;
	},
	createSelectionModel : function() {
		var me = this;
		
		me.sm = Ext.create('Ext.selection.CheckboxModel');
		
		me.sm.on('selectionchange', function(sm, selected, eOpts) {
			var btnRemove = me.getDockedComponent('group-toptoolbar').getComponent('btnGroupRemove');
			btnRemove.setDisabled(selected.length == 0);
		});
		
		return me;
	},
	createTopToolbar : function() {
		var me = this;
		
		me.tbar = Ext.create('Ext.toolbar.Toolbar', {
			id : 'group-toptoolbar',
			items : [{
	        	text : '추가',
	        	iconCls : 'icon-group-add',
	        	handler : function() {
	        		// create a model instance
	                var r = Ext.create('Group', {
	                	name : null,
	                	gid : 3001,
	                    members : []
	                });
	                me.getStore().insert(0, r);
	                
	                me.cellEditor.startEditByPosition({row: 0, column: 1});
	        	}
	        }, '-', {
	        	id : 'btnGroupRemove',
	        	disabled : true,
	        	text : '삭제',
	        	iconCls : 'icon-group-delete',
	        	scope : this,
	        	handler : function() {
	        		var self = this;
	        		var sm = self.getSelectionModel();
	        		
	        		var names = new Array();
	        		var records = sm.getSelection();
	        		
	        		Ext.each(records, function(r) {
	        			var name = r.get('name');
	        			
	        			if(!name) {
	        				self.getStore().remove(r);
	        			} else {
	        				names.push(name);
	        			}
	        		});
	        		
	        		if(names.length == 0) {
	        			return;
	        		}
	        		
	        		Ext.Ajax.request({
						url : CONTEXT_PATH + '/group/remove',
						params : names,
						success : function(response) {
							var json = Ext.decode(response.responseText);
							
							if(json.success == true) {
								self.getStore().load();
							} else {
								Ext.Msg.alert('삭제 실패', json.error.message);
							}
						}
					});
	        		
	        	}
	        }]
		});
		
		return this.tbar;
	},
	createBottomToolbar : function() {
		var me = this;
		
		me.bbar = Ext.create('Ext.toolbar.Paging', {
			store : me.store,
			dock : 'bottom',
			displayInfo : true,
			plugins : [{
				ptype: 'pagingtoolbarresizer', 
				options : [ 25, 50, 100 ]
			}]
		});
		
		return me.bbar;
	},
	createFilterBar : function() {
		this.filterBar = Ext.create('Ext.ux.grid.FilterBar', {
			renderHidden : false,
			showShowHideButton : false
		});
		
		return this.filterBar;
	},
	createCellEditor : function() {
		var me = this;
		
		me.cellEditor = Ext.create('Ext.grid.plugin.CellEditing', {
			scopt : me,
			listeners : {
				edit : function(editor, e, eOpts) {
					var record = e.record;
					
					if(!record.get('name') && !record.get('gid')) {
						return;
					}
					
					var data = Ext.clone(record.data);
					delete data['members'];
					data[e.field] = e.value;
					
					// 키가 되는 값이 변경 된 경우
					if(e.field == 'name') {
						data['originalName'] = e.originalValue;
					}
					
					Ext.Ajax.request({
						url : CONTEXT_PATH + '/group/save',
						params : data,
						success : function(response) {
							var json = Ext.decode(response.responseText);
							
							if(json.success) {
								record.commit();
							} else {
								Ext.Msg.alert('저장 실패', json.error.message);
								record.reject();
							}
						}
					});
					
				}
			}
		});
		
		return me.cellEditor;
	}
});