Ext.define('Proftpd.HistoryGrid', {
	id : 'Proftpd.HistoryGrid',
	// 트리 그리드 상속 받음
	extend : 'Ext.tree.Panel',
	uses : [ 'Ext.ux.PagingToolbarResizer', 'Ext.ux.grid.FilterBar',
			'Ext.ux.grid.feature.MultiSorting' ],
	pageSize : 25,
	constructor : function() {
		var me = this;

		// 모델 생성
		me.createModel();
		// 스토어 생성
		me.createStore();
		// 필터파 생성
		me.createFilterBar();
		// 아래쪽 툴바(페이징) 생성
		me.createBottomToolbar();
		
		Ext.apply(me, {
			title : '이력 조회',
			store : me.store,
			viewConfig : {
				stripeRows : true
			},
			collapsible : false,
			useArrows : true,
			rootVisible : false,
			// 필터바 플러그인
			plugins : [ this.filterBar ],
			// 다중 정렬 플러그인
			features : [ Ext.create('Ext.ux.grid.feature.MultiSorting') ],
			// 컬럼 목록
			columns : [ {
				text : '번호',
				dataIndex : 'seq',
				width : 60,
				align : 'right',
				hidden : true
			}, {
				flex : 1,
				minWidth : 200,
				xtype : 'treecolumn',
				text : '파일명',
				dataIndex : 'fileName',
				renderer : function(value) {
					return '<span title="' + value + '">' + value + '</span>';
				},
				filter : {
					type : 'string'
				}
			}, {
				align : 'right',
				text : '용량',
				dataIndex : 'bytes',
				renderer : function(value) {
					if (value > 0) {
						return Ext.util.Format.fileSize(value);
					}
				}
			}, {
				width : 80,
				text : '명령어',
				dataIndex : 'command',
				hidden : true
			}, {
				width : 100,
				text : '명령어 설명',
				dataIndex : null,
				hidden : true,
				renderer : this.commandNameRenderer
			}, {
				width : 80,
				text : '사용자',
				dataIndex : 'user',
				filter : {
					type : 'string'
				}
			}, {
				width : 110,
				text : '접속 아이피',
				dataIndex : 'clientIp',
				filter : {
					type : 'string'
				}
			}, {
				width : 160,
				text : '등록일시',
				dataIndex : 'created',
				renderer : Ext.util.Format.dateRenderer('Y-m-d A h:i:s')
			} ]
		});

		// render
		me.callParent(arguments);

	},
	// private
	commandNameRenderer : function(value, meta, record) {
		var command = record.get('command');
		var commandName;

		switch (command) {
		case 'DELE':
			commandName = '파일 삭제';
			break;
		case 'MKD':
			commandName = '폴더 생성';
			break;
		case 'RETR':
			commandName = '다운로드';
			break;
		case 'RMD':
			commandName = '폴더 삭제';
			break;
		case 'RNFR':
			commandName = '이름 변경전';
			break;
		case 'RNTO':
			commandName = '이름 변경후';
			break;
		case 'STOR':
			commandName = '업로드';
			break;
		default:
			commandName = 'N/A';
			break;
		}

		return commandName;
	},
	/*
	 * HistoryGrid의 스토어에서 사용할 모델을 정의한다.
	 */
	createModel : function() {
		// model
		Ext.define('History', {
			extend : 'Ext.data.Model',
			idProperty : 'seq',
			fields : [ {
				name : 'seq',
				type : 'int'
			}, {
				name : 'user',
				type : 'string'
			}, {
				name : 'command',
				type : 'string'
			}, {
				name : 'commandName',
				type : 'string'
			}, {
				name : 'fileName',
				type : 'string'
			}, {
				name : 'bytes',
				type : 'int'
			}, {
				name : 'clientIp',
				type : 'string'
			}, {
				name : 'created',
				type : 'date',
				convert : function(value) {
					// 서버단에서 Jackson 을 이용하여 json 을 받으면
					// java.util.Date 클래스는 getTime() 값이 넘어온다.
					// long 타입으로 데이터가 넘어온다
					return new Date(value);
				}
			}, {
				name : 'children'
			}, {
				name : 'leaf',
				mapping : 'children',
				type : 'boolean',
				convert : function(value) {
					// leaf가 true 이면 하위 노드가 없는 것
					return value == 0;
				}
			}, {
				name : 'iconCls',
				mapping : 'command',
				type : 'string',
				convert : function(value) {
					try {
						return 'icon-' + value.toLowerCase();
					} catch (e) {
						
					}
				}
			} ]
		});
	},
	/*
	 * 스토어 생성
	 */
	createStore : function() {
		var me = this;
		
		me.store = Ext.create('Ext.ux.data.TreePagingStore', {
			model : 'History',
			proxy : {
				type : 'ajax',
				url : CONTEXT_PATH + '/history/list',
				reader : Ext.create('Ext.data.reader.Json'),
				actionMethods : {
					read : 'post'
				}
			},
			root : {
				expanded : true,
				text : null,
				children : []
			},
			defaultRootId : null,
			defaultRootProperty : 'results',
			pageSize : me.pageSize,
			remoteSort : true,
			remoteFilter : true,
			nodeParam : 'parent'
		});
		
		me.store.on({
			beforeload : function() {
				if (me.body && !me.body.isMasked()) {
					me.body.mask('로딩중...');
				}
			},
			load : function() {
				if (me.body && me.body.isMasked()) {
					me.body.unmask();
				}
			}
		});
		
		return me.store;
	},
	/*
	 * 필터바 생성
	 */
	createFilterBar : function() {
		this.filterBar = Ext.create('Ext.ux.grid.FilterBar', {
			renderHidden : false,
			showShowHideButton : false
		});

		return this.filterBar;
	},
	/*
	 * 페이징 툴바 생성
	 */
	createBottomToolbar : function() {
		var me = this;
		
		me.bbar = Ext.create('Ext.ux.toolbar.TreePaging', {
			grid : me,
			displayInfo : true,
			inputItemWidth : 40,
			plugins : [ {
				ptype : 'pagingtoolbarresizer',
				options : [ 10, 25, 50, 100 ]
			} ]
		});

		return me.bbar;
	}
});