Ext.define('Proftpd.LoginWindow', {
	id : 'Proftpd.LoginWindow',
	extend : 'Ext.window.Window',
	constructor : function() {
		
		var me = this;
		
		Ext.apply(me, {
			title : '로그인',
			width : 300,
			modal : true,
			closable : false,
			layout : 'fit',
			items : me.createForm(),
			buttons : [{
				text : '로그인',
				handler : me.doSubmit,
				scope : me
			}]
		});
	
		// create
		me.callParent(arguments);
		
		me.on('show', me.initFocus);
	},
	createForm : function() {
		var me = this;

		var required = '<span style="color:red;font-weight:bold" data-qtip="Required">*</span>';
		
		me.form = Ext.create('Ext.form.Panel', {
			bodyPadding : 5,
			defaultType: 'textfield',
			border : false,
			items : [{
				fieldLabel : '아이디',
				afterLabelTextTpl : required,
				id : 'username',
				name : 'username',
				allowBlank : false,
				listeners : {
					specialkey : function(field, e) {
						if (e.getKey() == e.ENTER) {
							me.doSubmit();
						}
					}
				}
			}, {
				fieldLabel : '비밀번호',
				afterLabelTextTpl : required,
				id : 'pssword',
				name : 'password',
				inputType : 'password',
				allowBlank : false,
				listeners : {
					specialkey : function(field, e) {
						if (e.getKey() == e.ENTER) {
							me.doSubmit();
						}
					}
				}
			}]
		});
		
		return me.form;
	},
	doSubmit : function(btn, event) {
		me = this; // window

		me.el.mask('로그인중...');
		
		var form = me.form.getForm();

		if(form.isValid()) {
		
			form.submit({
				url : CONTEXT_PATH + '/login',
				success : function(form, action) {
					
					// 이 로그인창을 띄울때 넘겨준 데이터
					var conn = me.req.conn;
					var options = me.req.options;

					// 로그인창 닫기
					me.close();
					
					// 인증이 된 후 재로딩
					options.params = Ext.decode(options.params);
					conn.request(options);
				},
				failure : function(form, action) {
					switch (action.failureType) {
		            case Ext.form.action.Action.CONNECT_FAILURE:
		                Ext.Msg.alert('실패', 'AJAX 통신에 실패하였습니다.');
		                break;
		            case Ext.form.action.Action.SERVER_INVALID:
		            	var json = Ext.decode(action.response.responseText);
		            	Ext.Msg.alert('Failure', json.error.message, function() {
		            		me.initFocus();
		            	});
					}
					
					me.el.unmask();
				}
			});
			
		}
		
	},
	initFocus : function() {
		// 0.5초 후에 아이디 입력필드 포커스 이동
		var me = this;
		var field = me.form.getComponent('username');
		
		var task = new Ext.util.DelayedTask(function(){
			field.focus();
		});
		
		task.delay(300);
	}
});