Ext.define('Proftpd.MembersGrid', {
	id : 'Proftpd.MembersGrid',
	extend : 'Ext.grid.Panel',
	uses : [
		'Ext.ux.PagingToolbarResizer',
		'Ext.ux.CheckColumn',
		'Ext.ux.form.field.ClearButton',
		'Ext.ux.grid.feature.MultiSorting'
	],
	constructor : function() {
		var me = this;

		// model
		Ext.define('UserInGroup', {
			extend : 'Ext.data.Model',
			idProperty : 'id',
			fields : [ 'id', 'userid', 'groupName']
		});
		
		// store
		me.createStore();
		// top toolbar
		me.createTopToolbar();
		// bottom bar
		me.createBottomToolbar();
		// selection model
		me.createSelectionModel();
		
		Ext.apply(me, {
			border : false,
			store : me.store,
			viewConfig: {
	            stripeRows: true
	        },
	        selModel: me.sm,
			columns : [ {
				flex : 1,
				text : '사용자 아이디',
				dataIndex : 'userid',
				filter : {
					type : 'string'
				},
				field: {
	                allowBlank: false
	            }
			}, {
				flex : 1,
				text : '현재 그룹',
				dataIndex : 'groupName',
				renderer : function(value) {
					if(me.groupName == value) {
						return '<span style="color:blue">' + value + '</span>';
					} else {
						return value;
					}
				}
			}]
		});

		// create
		me.callParent(arguments);
	},
	/*
	 * 사용자 목록을 불러온다.
	 */
	load : function(groupName) {
		var me = this;
		
		me.groupName = groupName;
		
		me.getStore().load({
			params : {
				groupName : me.groupName
			}
		});
		
		if(!me.originalTitle) {
			me.originalTitle = me.title;
		}
		
		me.setTitle(me.originalTitle + ' (그룹: ' + me.groupName + ')');
	},
	/*
	 * 초기화 
	 */
	clearData : function() {
		var me = this;
		
		me.setTitle(me.originalTitle);
		me.getStore().removeAll();
	},
	createStore : function() {
		var me = this;
		
		me.store = Ext.create('Ext.data.Store', {
			model : 'UserInGroup',
			pageSize : 50,
			proxy : {
				type : 'ajax',
				url : CONTEXT_PATH + '/member/list/group',
				reader : {
					type : 'json',
					root : 'results',
					totalProperty : 'total'
				},
				actionMethods : {
					read : 'post'
				}
			},
			remoteSort : true,
			remoteFilter : true
		});
		
		me.store.on({
			beforeload : function() {
				if(!me.body.isMasked()) {
					me.body.mask('로딩중...');
				}
			},
			load : function() {
				if(me.body.isMasked()) {
					me.body.unmask();
				}
			}
		});
		
		return me.store;
	},
	createSelectionModel : function() {
		var me = this;
		
		me.sm = Ext.create('Ext.selection.CheckboxModel');
		
		me.sm.on('selectionchange', function(sm, selected) {
			var tbar = me.getDockedComponent('members-toptoolbar');
			
			var btnJoin = tbar.getComponent('btnJoin');
			var btnLeave = tbar.getComponent('btnLeave');
			
			btnJoin.setDisabled(selected.length == 0);
			btnLeave.setDisabled(selected.length == 0);
		});
	},
	createTopToolbar : function() {
		var me = this;
		
		me.tbar = Ext.create('Ext.toolbar.Toolbar', {
			id : 'members-toptoolbar',
			items : [{
				id : 'btnJoin',
				disabled : true,
	        	text : '포함',
	        	iconCls : 'icon-group-add',
	        	listeners : {
	        		scope : me,
	        		click : me.joinUser,
	        		mouseover : function(btn) {
	        			if(this.groupName) {
	        				btn.setTooltip('\'' + me.groupName + '\' 그룹에 포함시킵니다.');
	        			}
	        		}
	        	}
			}, '-', {
				id : 'btnLeave',
				disabled : true,
	        	text : '제외',
	        	iconCls : 'icon-group-delete',
	        	scope : this,
	        	handler : me.leaveUser,
	        	scope : me,
	        	tooltip : '그룹에서 제외합니다.'
			}]
		});
	},
	createBottomToolbar : function() {
		var me = this;
		
		me.bbar = Ext.create('Ext.toolbar.Paging', {
			store : me.store,
			dock : 'bottom',
			displayInfo : true
		});
		
		return me.bbar;
	},
	/*
	 * 선택한 사용자를 그룹에 포함시킨다.
	 */
	joinUser : function() {
		var me = this;
		var sm = me.getSelectionModel();
		
		if(sm.getCount() == 0) {
			return;
		}
		
		var records = sm.getSelection();
		
		var users = [];
		Ext.each(records, function(r) {
			users.push(r.get('userid'));
		});
		
		Ext.Ajax.request({
			url : CONTEXT_PATH + '/group/join',
			params : {
				groupName : me.groupName,
				users : users
			},
			success : function(response) {
				var json = Ext.decode(response.responseText);
				
				if(json.success == true) {
					me.getStore().load();
				} else {
					Ext.Msg.alert('포함 실패', json.error.message);
				}
			}
		});
	},
	/*
	 * 선택한 사용자를 그룹에 제외시킨다.
	 */
	leaveUser : function() {
		var me = this;
		var sm = me.getSelectionModel();
		
		if(sm.getCount() == 0) {
			return;
		}
		
		var records = sm.getSelection();
		
		var users = [];
		Ext.each(records, function(r) {
			if(r.get('groupName')) {
				users.push(r.get('userid'));
			}
		});

		Ext.Ajax.request({
			url : CONTEXT_PATH + '/group/leave',
			params : users,
			success : function(response) {
				var json = Ext.decode(response.responseText);
				
				if(json.success == true) {
					me.getStore().load();
				} else {
					Ext.Msg.alert('제외 실패', json.error.message);
				}
			}
		});
	}
});