Ext.define('Proftpd.UserGrid', {
	id : 'Proftpd.UserGrid',
	extend : 'Ext.grid.Panel',
	uses : [
		'Ext.ux.PagingToolbarResizer',
		'Ext.ux.CheckColumn',
		'Ext.ux.form.field.ClearButton',
		'Ext.ux.grid.feature.MultiSorting'
	],
	constructor : function() {
		var me = this;

		var roles = Ext.create('Ext.data.Store', {
			fields : ['value', 'name'],
			data : [{
				name : '사용자',
				value : 'ROLE_USER'
			}, {
				name : '관리자',
				value : 'ROLE_ADMIN'
			}]
		});
		
		// model
		me.createModel();
		// store
		me.createStore();
		// selection model
		me.createSelectionModel();
		// top toolbar
		me.createTopToolbar();
		// bottom bar
		me.createBottomToolbar();
		// filter bar plugin
		me.createFilterBar();
		// cell editing
		me.createCellEditor();
		
		Ext.apply(me, {
			title : '사용자 관리',
			store : me.store,
			viewConfig: {
	            stripeRows: true
	        },
	        selModel: me.sm,
			columns : [ {
				flex : 1,
				text : '아이디',
				dataIndex : 'userid',
				width : 100,
				filter : {
					type : 'string'
				},
				field: {
	                allowBlank: false
	            }
			}, {
				text : '비밀번호',
				dataIndex : 'passwd',
				width : 100,
				hidden : true,
				renderer : function(value) {
					if(value == null) {
						return '';
					}
					
					var str = '';
					for(var i = 0 ; i < value.length ; i++) {
						str += '*';
					}
					
					return str;
				},
				field : {
					inputType : 'password',
					allowBlank : false
				}
			}, {
				text : '이메일',
				dataIndex : 'email',
				flex : 2,
				minWidth : 150,
				renderer : function(value) {
					return '<a href="mailto:' + value + '">' + value + '</a>';
				},
				filter : {
					type : 'string'
				},
				field: {
	                allowBlank: false,
	                vtype : 'email'
	            }
			}, {
				text : '사용자 식별자',
				dataIndex : 'uid',
				hidden : true,
				editor : {
					xtype : 'numberfield'
				},
				filter : {
					type : 'int'
				}
			}, {
				text : '그룹 식별자',
				dataIndex : 'gid',
				hidden : true,
				editor : {
					xtype : 'numberfield'
				},
				filter : {
					type : 'int'
				}
			}, {
				text : '쉘',
				flex : 1,
				minWidth : 150,
				dataIndex : 'shell',
				hidden : true,
				filter : {
					type : 'string'
				}
			}, {
				text : '홈 디렉터리',
				flex : 1,
				minWidth : 150,
				hidden : true,
				dataIndex : 'homedir',
				filter : {
					type : 'string'
				}
			}, {
				width : 80,
				text : '접속수',
				dataIndex : 'count',
				align : 'right',
				renderer : Ext.util.Format.numberRenderer('0,000')
			}, {
				width : 160,
				text : '마지막 접속일시',
				dataIndex : 'accessed',
				renderer : function(value) {
					if (value == null) {
						return '-';
					}

					return Ext.util.Format.date(value, 'Y-m-d A h:i:s');
				},
				filter : {
					type : 'date',
					format : 'Y-m-d'
				}
			}, {
				width : 70,
				text : '허용',
				dataIndex : 'valid',
				align : 'center',
				renderer : function(value) {
					var ret = value == true ? '허용' : '차단';
					
					if (value == false) {
						ret = '<sapn style="color: #ff0000">' + ret + '</span>';
					}
					
					return ret;
				},
				field : {
					xtype : 'combo',
					editable : false,
					store : [[true, '허용'], [false, '차단']]
				}
			}, {
				width : 90,
				text : '권한',
				dataIndex : 'authority',
				renderer : function(v) {
					switch (v) {
					case 'ROLE_USER':
						return '사용자';
						break;
					case 'ROLE_ADMIN':
						return '관리자';
						break;
					default:
						return v;
						break;
					}
				},
				field : {
					xtype: 'combo',
					queryMode: 'local',
					forceSelection: true,
					editable: false,
					triggerAction: 'all',
					store : roles,
					displayField : 'name',
					valueField : 'value'
				}
			} ],
			// 순서 주의!
			plugins : [ this.filterBar, this.cellEditor ],
			features : [ Ext.create('Ext.ux.grid.feature.MultiSorting') ]
		});

		// create
		me.callParent(arguments);
		
		me.on({
			scope : this,
			render : function() {
				me.store.load();
			}
		});

	},
	createModel : function() {
		// model
		Ext.define('User', {
			extend : 'Ext.data.Model',
			idProperty : 'id',
			fields : [ 'id', 'userid', 'passwd', 'email', {
				name : 'uid',
				type : 'int'
			}, {
				name : 'gid',
				type : 'int'
			}, 'shell', 'homedir', {
				name : 'count',
				type : 'int'
			}, {
				name : 'accessed',
				type : 'date',
				convert : function(value) {
					if (value == null) {
						return null;
					}

					// long -> date
					return new Date(value);
				}
			}, {
				name : 'valid',
				type : 'bool'
			}, 'authority' ]
		});
	},
	createStore : function() {
		var me = this;
		
		me.store = Ext.create('Ext.data.Store', {
			model : 'User',
			pageSize : 50,
			proxy : {
				type : 'ajax',
				url : CONTEXT_PATH + '/user/list',
				reader : {
					type : 'json',
					root : 'results',
					totalProperty : 'total'
				},
				actionMethods : {
					read : 'post'
				}
			},
			remoteSort : true,
			remoteFilter : true
		});
		
		me.store.on({
			beforeload : function() {
				if(!me.body.isMasked()) {
					me.body.mask('로딩중...');
				}
			},
			load : function() {
				if(me.body.isMasked()) {
					me.body.unmask();
				}
			}
		});
		
		return me.store;
	},
	createSelectionModel : function() {
		var me = this;
		
		me.sm = Ext.create('Ext.selection.CheckboxModel');
		
		me.sm.on('selectionchange', function(sm, selected, eOpts) {
			var btnRemove = me.getDockedComponent('user-toptoolbar').getComponent('btnUserRemove');
			btnRemove.setDisabled(selected.length == 0);
		});
		
		return me;
	},
	createTopToolbar : function() {
		var me = this;
		
		me.tbar = Ext.create('Ext.toolbar.Toolbar', {
			id : 'user-toptoolbar',
			items : [{
	        	id : 'btnUserAdd',
	        	text : '추가',
	        	iconCls : 'icon-vcard-add',
	        	handler : function() {
	        		// show password column
	        		Ext.each(me.columns, function(column) {
	        			if(column.dataIndex == 'passwd') {
	        				if(column.isHidden()) {
	        					column.show();
	        				}
	        				
	        				return false;
	        			}
	        		});
	        		
	        		// create a model instance
	                var r = Ext.create('User', {
	                	passwd : null,
	                	accessed : null,
	                    valid : true,
	                    authority : 'ROLE_USER'
	                });
	                me.getStore().insert(0, r);
	                
	                me.cellEditor.startEditByPosition({row: 0, column: 1});
	        	}
	        }, '-', {
	        	id : 'btnUserRemove',
	        	text : '삭제',
	        	iconCls : 'icon-vcard-delete',
	        	disabled : true,
	        	handler : function() {
	        		var sm = me.getSelectionModel();
	        		
	        		var ids = new Array();
	        		var records = sm.getSelection();
	        		
	        		Ext.each(records, function(r) {
	        			var id = r.get('id');
	        			
	        			if(!id) {
	        				me.getStore().remove(r);
	        			} else {
	        				ids.push(id);
	        			}
	        		});
	        		
	        		if(ids.length == 0) {
	        			return;
	        		}
	        		
	        		Ext.MessageBox.show({
	        			title : '사용자 삭제',
	        			msg : '선택한 사용자를 삭제하시겠습니까?',
	        			buttons : Ext.MessageBox.YESNO,
	        			fn : function(buttonId, text, opt) {
	        				if(buttonId == 'yes') {
	        					// request ajax
	        					Ext.Ajax.request({
	        						url : CONTEXT_PATH + '/user/remove',
	        						params : ids,
	        						success : function(response) {
	        							var json = Ext.decode(response.responseText);
	        							
	        							if(json.success == true) {
	        								me.getStore().load();
	        							} else {
	        								Ext.Msg.alert('삭제 실패', json.error.message);
	        							}
	        						}
	        					});
	        					
	        				}
	        			}
	        		});
	        		
	        	}
	        }]
		});
		
		return me.tbar;
	},
	createBottomToolbar : function() {
		var me = this;
		
		me.bbar = Ext.create('Ext.toolbar.Paging', {
			store : me.store,
			dock : 'bottom',
			displayInfo : true,
			plugins : [{
				ptype: 'pagingtoolbarresizer', 
				options : [ 25, 50, 100 ]
			}]
		});
		
		return me.bbar;
	},
	createFilterBar : function() {
		this.filterBar = Ext.create('Ext.ux.grid.FilterBar', {
			renderHidden : false,
			showShowHideButton : false
		});
		
		return this.filterBar;
	},
	createCellEditor : function() {
		var me = this;
		
		me.cellEditor = Ext.create('Ext.grid.plugin.CellEditing', {
			scopt : me,
			listeners : {
				edit : function(editor, e, eOpts) {
					var record = e.record;
					
					if (!record.get('id')) { // 추가
						// 필수로 입력해야 하는 부분 검사
						if (!record.get('userid') || !record.get('passwd') || !record.get('email')) {
							return;
						}
					} else { // 수정
						if(e.originalValue == e.value) {
							return;
						}
					}
					
					var data = Ext.clone(record.data);
					delete data['count'];
					delete data['accessed'];
					data[e.field] = e.value;
					
					Ext.Ajax.request({
						url : CONTEXT_PATH + '/user/save',
						params : data,
						success : function(response) {
							var json = Ext.decode(response.responseText);
							
							if(json.success) {
								if(!record.get('id')) {
									record.set('id', json.user.id);
									record.commit();
								}
								
								Ext.each(me.columns, function(column) {
									if(column.dataIndex == 'passwd') {
										if(!column.isHidden()) {
											column.hide();
										}
										
										return false;
									}
								});
							} else {
								Ext.Msg.alert('저장 실패', json.error.message);
								record.reject();
							}
						},
						callback : function(options, success, response) {
							record.set('passwd', null);
							record.commit();
						}
					});
				}
			}
		});
		
		return me.cellEditor;
	}
});